<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
	xmlns="http://www.w3.org/1999/xhtml"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:ws="http://www.ebenoit.info/data"
	xmlns:wsf="http://www.ebenoit.info/ext">


	<!-- *********************************************************************************************** -->
	<!-- * HISTORY BOX AND COMPLETE HISTORY                                                            * -->
	<!-- *********************************************************************************************** -->

	<!-- Generate the history's box
	   -
	   - This template generates a side box for the last 5 entries of the history and calls the history
	   - contents generator.
	  -->
	<xsl:template name="generate-history-box">
		<xsl:call-template name="side-box-template">
			<xsl:with-param name="id">history-box</xsl:with-param>
			<xsl:with-param name="title">
				<xsl:call-template name="get-i18n-string">
					<xsl:with-param name="id">history</xsl:with-param>
				</xsl:call-template>
			</xsl:with-param>
			<xsl:with-param name="contents">
				<xsl:call-template name="generate-short-history" />
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>


	<!-- Short hisotry generator
	   -
	   - This template generates the contents of the history box.
	  -->
	<xsl:template name="generate-short-history">
		<xsl:choose>

			<!-- Is the history in a separate file? -->
			<xsl:when test="$c-block/ws:history/@datafile">
				<xsl:for-each select="document($c-block/ws:history/@datafile)/ws:history">
					<xsl:call-template name="process-short-history" />
				</xsl:for-each>
			</xsl:when>

			<!-- Is the history inlined? -->
			<xsl:when test="$c-block/ws:history/ws:entry">
				<xsl:for-each select="$c-block/ws:history">
					<xsl:call-template name="process-short-history" />
				</xsl:for-each>
			</xsl:when>

			<!-- No history -->
			<xsl:otherwise>
				<div class="line-content">
					<xsl:call-template name="get-i18n-string">
						<xsl:with-param name="id">empty-history</xsl:with-param>
					</xsl:call-template>
				</div>
			</xsl:otherwise>

		</xsl:choose>
	</xsl:template>


	<!-- Short history processor
	   -
	   - This template generates the contents of the history box from a ws:history
	   - tag.
	  -->
	<xsl:template name="process-short-history">
		<!-- Generate the contents -->
		<div id="history-contents">
			<xsl:for-each select="ws:entry[ position() &lt;= 5 ]">
				<xsl:call-template name="short-history-template" />
			</xsl:for-each>
		</div>

		<!-- If there is a page with the whole history, link to it -->
		<xsl:if test="$c-block/ws:pages/ws:page/ws:complete-history">
			<xsl:variable name="ch-page" select="$c-block/ws:pages/ws:page/ws:complete-history/../@id" />
			<xsl:variable name="ch-title">
				<xsl:call-template name="get-i18n-string">
					<xsl:with-param name="id">complete-history</xsl:with-param>
				</xsl:call-template>
			</xsl:variable>

			<div id="history-more">
				<a>
					<xsl:attribute name="href"><xsl:value-of select="concat( $scriptPath , '/' , $lang , '/' , $ch-page )" /></xsl:attribute>
					<xsl:attribute name="title"><xsl:value-of select="$ch-title" /></xsl:attribute>
					<xsl:value-of select="$ch-title" />
				</a>
			</div>
		</xsl:if>
	</xsl:template>


	<!-- Complete history tag handler
	   -
	   - This template will transform the <ws:complete-history> tag into the complete history
	   - of the site's changes.
	  -->
	<xsl:template match="ws:complete-history">
		<xsl:call-template name="complete-history-container-template">
			<xsl:with-param name="contents">
				<xsl:call-template name="generate-complete-history" />
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>


	<!-- Complete history generator
	   -
	   - This template will generate the complete history by looking for the source of the
	   - entries.
	  -->
	<xsl:template name="generate-complete-history">
		<xsl:choose>

			<!-- Is the history in a separate file? -->
			<xsl:when test="$c-block/ws:history/@datafile">
				<xsl:for-each select="document($c-block/ws:history/@datafile)/ws:history">
					<xsl:call-template name="process-complete-history" />
				</xsl:for-each>
			</xsl:when>

			<!-- Is the history inlined? -->
			<xsl:when test="$c-block/ws:history/ws:entry">
				<xsl:for-each select="$c-block/ws:history">
					<xsl:call-template name="process-complete-history" />
				</xsl:for-each>
			</xsl:when>

			<!-- No history -->
			<xsl:otherwise>
				<xsl:call-template name="complete-history-empty-template">
					<xsl:with-param name="message">
						<xsl:call-template name="get-i18n-string">
							<xsl:with-param name="id">empty-history</xsl:with-param>
						</xsl:call-template>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:otherwise>

		</xsl:choose>
	</xsl:template>


	<!-- Short history processor
	   -
	   - This template generates the contents of the history box from a ws:history
	   - tag.
	  -->
	<xsl:template name="process-complete-history">
		<xsl:for-each select="ws:entry">
			<xsl:call-template name="complete-history-entry-template" />
		</xsl:for-each>
	</xsl:template>


</xsl:stylesheet>