# -*- coding: utf-8 -*-

"""

PySSE v0.2.1

Copyright (C) 2009 Emmanuel Benoît <ebenoit@ebenoit.info>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

"""

from mod_python import apache
from os.path import dirname
from lxml import etree
import re


def get_languages( dataFile ):

	lFragments = dataFile.xpath( '/ws:data/ws:contents' ,
			namespaces = { 'ws': 'http://www.ebenoit.info/data' } )

	languages = [ node.get( 'lang' ) for node in lFragments ]
	if not languages:
		languages = [ 'en' ]

	return languages


def guess_language( acceptString , languages ):
	acceptString = re.sub( r'[^a-zA-Z0-9\.\-;,=]' , '' , acceptString )
	if len( acceptString ) == 0:
		return languages[ 0 ]

	# Build a list using the different weights as keys
	accepted = { }
	aList = acceptString.split( ',' )

	for lString in aList:

		if lString == '':
			continue

		lData = lString.split( ';q=' )
		if len( lData ) == 1:
			lData.append( '1.0' )

		lStr = lData[ 0 ]
		lWeight = float( lData[ 1 ] ) * 1000

		if lWeight in accepted:
			accepted[ lWeight ].append( lStr )
		else:
			accepted[ lWeight ] = [ lStr ]

	# Iterate for all languages in the weight-specific order
	weights = accepted.keys( );
	weights.sort( );
	for lWeight in reversed( weights ) :
		for lStr in accepted[ lWeight ]:
			if lStr in languages:
				return lStr

	return languages[ 0 ]



def handler( req ):

	# Find the script's base URI and physical path
	sPath = req.parsed_uri[ apache.URI_PATH ]
	if sPath.find( 'index.py' ) == -1:
		sPath = sPath + 'index.py'
	if req.path_info:
		sPath = sPath[ 0 : -len( req.path_info ) ]
	basePath = dirname( sPath )
	if basePath[ len( basePath ) - 1 ] != '/' :
		basePath = basePath + '/'
	dataPath = dirname( req.finfo[apache.FINFO_FNAME] ) + '/data/'

	# Extract requested language and page from the path
	lang = ""
	page = "main"
	if req.path_info:
		splitPath = req.path_info[ 1 : ].split( '/' )
		if len( splitPath ) == 1:
			page = splitPath.pop(0)
		elif len( splitPath ) >= 2:
			lang = splitPath.pop(0)
			page = splitPath.pop(0)

	# Check if the page name is valid
	vPageExp = re.compile( '^[a-z][a-z0-9\-]*$' , re.IGNORECASE )
	if not vPageExp.match( page ):
		page = "main"

	# Some useful variables for handling namespaces
	xhtml_ns = 'http://www.w3.org/1999/xhtml'
	xhtml = '{%s}' % xhtml_ns
	ws_ns = 'http://www.ebenoit.info/data'
	ws = '{%s}' % ws_ns
	nsmap = { None : xhtml_ns , 'ws' : ws_ns }

	# Define the extension function containers
	dataFile = None
	qns = { 'ws' : ws_ns }
	def xsl_tfunction_container( d_func ):
		def f( useless , *args ):
			try:
				l = d_func( *args )
				x = etree.Element( ws + 'data' , nsmap = nsmap )
				x.extend( l )
				return [ x ]
			except Exception, e:
				hdr = etree.Element( xhtml + 'h2' , nsmap = nsmap )
				hdr.text = 'Internal error'
				p = etree.Element( xhtml + 'p' , nsmap = nsmap )
				p.text = str( e )
				return [ hdr , p ]
		return f
	def xsl_sfunction_container( d_func ):
		def f( useless , *args ):
			try:
				return unicode( d_func( *args ) )
			except Exception, e:
				return unicode( 'Internal error: ' + e.text )
		return f


	# Function that gets a string's translation
	translations = {}
	translation_files = {}
	def get_i18n_string( string_id , lang_id ):

		# Is it cached?
		if ( string_id , lang_id ) in translations:
			return translations[ ( string_id , lang_id ) ]

		# Look in the main configuration file
		xpath_root = "/ws:data/ws:contents[ @lang = $lid ]/ws:strings"
		xpath_expr = xpath_root + "/ws:string[ @id = $sid ]"
		r = dataFile.xpath( xpath_expr ,
							lid = lang_id ,
							sid = string_id ,
							namespaces = qns )
		if len( r ) >= 1:
			translations[ ( string_id , lang_id ) ] = r[ 0 ].text
			return translations[ ( string_id , lang_id ) ]

		# Is there a separate data file?
		if lang_id not in translation_files:
			xpath_expr = xpath_root + "[ @datafile ]"
			r = dataFile.xpath( xpath_expr , lid = lang_id , namespaces = qns )
			if len( r ) < 1:
				return unicode( '(string not found)' )
			translation_files[ lang_id ] = etree.parse(
					dataPath + r[0].attrib[ 'datafile' ]
				)

		# Look in the separate file
		f = translation_files[ lang_id ]
		xpath_expr = "/ws:strings/ws:string[ @id = $sid ]"
		r = f.xpath( xpath_expr , sid = string_id , namespaces = qns )
		if len( r ) < 1:
			return unicode( '(string not found)' )
		translations[ ( string_id , lang_id ) ] = r[ 0 ].text
		return translations[ ( string_id , lang_id ) ]


	# Function that gets a page's title
	titles = {}
	def get_page_title( page_id , lang_id ):

		# Is it cached?
		if ( page_id , lang_id ) in titles:
			return titles[ ( page_id , lang_id ) ]

		# Look in the language-specific section
		xpath_expr = ( "/ws:data/ws:contents[ @lang = $lid ]/ws:pages"
					 + "/ws:page[ @id = $pid and @title ]" )
		r = dataFile.xpath( xpath_expr ,
							lid = lang_id ,
							pid = page_id ,
							namespaces = qns )
		if len( r ) >= 1:
			titles[ ( page_id , lang_id ) ] = r[ 0 ].attrib[ 'title' ]
			return titles[ ( page_id , lang_id ) ]

		# Look in the sitemap
		xpath_expr = "/ws:data/ws:sitemap/ws:page[ @id = $pid and @title ]"
		r = dataFile.xpath( xpath_expr ,
							pid = page_id ,
							namespaces = qns )
		if len( r ) >= 1:
			titles[ ( page_id , lang_id ) ] = r[ 0 ].attrib[ 'title' ]
		else:
			titles[ ( page_id , lang_id ) ] = unicode( '(title not found)' )
		return titles[ ( page_id , lang_id ) ]


	# Function that reads a page's data
	page_data = {}
	def get_page_data( page_id , lang_id ):

		# Is it cached?
		if ( page_id , lang_id ) in page_data:
			return page_data[ ( page_id , lang_id ) ]

		# Find the page's record
		ns = { 'ws' : ws_ns }
		xpath_expr = ( '/ws:data/ws:contents[ @lang = $lang ]'
					   + '/ws:pages/ws:page[ @id = $pid ]' )
		page_lst = dataFile.xpath( xpath_expr ,
									lang = lang_id ,
									pid = page_id ,
									namespaces = qns )
		if len( page_lst ) != 1:
			hdr = etree.Element( xhtml + 'h2' , nsmap = nsmap )
			hdr.text = 'Error for page ' + page_id + ' in language ' + lang_id
			p = etree.Element( xhtml + 'p' , nsmap = nsmap )
			p.text = 'The page could not be found in the site\'s configuration.'
			return [ hdr , p ]
		page = page_lst[ 0 ]

		# Does it have a separate data file?
		if 'datafile' not in page.attrib:
			# Nope, return the tag's contents
			page_data[ ( page_id , lang_id ) ] = list( page )
			return page_data[ ( page_id , lang_id ) ]

		# Get the parameters that determine the path to the page's data file
		file_path = dataPath
		xpath_expr = '/ws:data/ws:page-data'
		r = dataFile.xpath( xpath_expr , namespaces = qns )
		if len( r ) == 1:
			file_path += r[ 0 ].text + '/'
		xpath_expr = '/ws:data/ws:contents[ @lang = $lang ]/@language-path'
		r = dataFile.xpath( xpath_expr , lang = lang_id , namespaces = qns )
		if len( r ) == 1:
			file_path += lang_id + '/'
		else:
			xpath_expr = '/ws:data/ws:contents[ @lang = $lang ]/@base-path'
			r = dataFile.xpath( xpath_expr , lang = lang_id ,
								namespaces = qns )
			if len( r ) == 1:
				file_path += r[ 0 ].text + '/'
		file_path += page.attrib[ 'datafile' ] + '.xml'

		# Get the data from the file
		page_file = etree.parse( file_path )
		page_data[ ( page_id , lang_id ) ] = list( page_file.getroot() )
		return page_data[ ( page_id , lang_id ) ]


	# Function that generates the breadcrumb
	def generate_breadcrumb( page_id , lang_id , omit_last , top_level_string ):

		# Find out the home page's identifier
		xpath_expr = '/ws:data/ws:sitemap'
		r = dataFile.xpath( xpath_expr , namespaces = qns )
		if 'home' in r[ 0 ].attrib:
			home_id = r[ 0 ].attrib[ 'home' ]
		else:
			home_id = 'main'

		if page_id == home_id:
			# This is the root, do we display the string?
			span = etree.Element( xhtml + 'span' , nsmap = nsmap )
			if top_level_string:
				span.text = get_i18n_string( 'breadcrumb-top' , lang_id )
			else:
				span.text = get_page_title( page_id , lang_id )
			return [ span ]

		# Get the breadcrumb separator
		xpath_expr = '/ws:data/ws:breadcrumb[ @separator ]'
		r = dataFile.xpath( xpath_expr , namespaces = qns )
		if len( r ) >= 1:
			bc_sep = r[ 0 ].attrib[ 'separator' ]
		else:
			bc_sep = unicode( ' ' )

		# Not at the root
		bc = [ ]
		current = page_id
		previous = ''
		while True:

			# Create the crumb
			if not( current == page_id and omit_last ):
				n_text = get_page_title( current , lang_id )
				if previous != page_id or not( omit_last ):
					span = etree.Element( xhtml + 'span' , nsmap = nsmap )
					if current == page_id:
						span.text = n_text
					else:
						span.text = bc_sep
					bc.insert( 0 , span )

				if current != page_id:
					# This is a link
					link = etree.Element( ws + 'link-to' , nsmap = nsmap )
					link.attrib[ 'page' ] = current
					link.text = n_text
					bc.insert( 0 , link )

			# End the loop if that was the home page
			if current == home_id:
				break

			# Get the previous page in the hierarchy
			previous = current
			xpath_expr = ( "/ws:data/ws:sitemap/ws:page"
						 + "[ @id = $pid and @parent ]" )
			r = dataFile.xpath( xpath_expr , pid = current , namespaces = qns )
			if len( r ) >= 1:
				current = r[ 0 ].attrib[ 'parent' ]
			else:
				current = home_id

		return bc

	# Register the functions
	ext_ns = etree.FunctionNamespace( 'http://www.ebenoit.info/ext' )
	ext_ns[ 'get-i18n-string' ] = xsl_sfunction_container( get_i18n_string )
	ext_ns[ 'get-page-title' ] = xsl_sfunction_container( get_page_title )
	ext_ns[ 'get-page-data' ] = xsl_tfunction_container( get_page_data )
	ext_ns[ 'generate-breadcrumb' ] = xsl_tfunction_container( generate_breadcrumb )

	# Read the data file
	dataFile = etree.parse( dataPath + 'site.xml' )

	# Find out which languages are supported
	languages = get_languages( dataFile )

	# Check if the selected language is valid
	if lang != '' and lang not in languages:
		lang = ''

	# If there is no specified language, try to guess it
	# according to Accept-Language
	if lang == '':
		if req.headers_in.has_key( 'Accept-Language' ):
			acceptString = req.headers_in[ 'Accept-Language' ]
			lang = guess_language( acceptString , languages )
		else:
			lang = languages[ 0 ]

	# Load the XSLT stylesheet
	transform = etree.XSLT( etree.parse( dataPath + 'site.xsl' ) )

	# Check if the client accepts application/xhtml+xml
	if req.headers_in.has_key( 'Accept' ):
		accept = re.split( '[;,]' , req.headers_in[ 'Accept' ].lower() )
		if 'application/xhtml+xml' in accept:
			req.content_type = 'application/xhtml+xml'
		else:
			req.content_type = 'text/html'
	else:
		req.content_type = 'text/html'

	req.write( str( transform( dataFile ,
			reqPage = "'" + page + "'" ,
			reqLang = "'" + lang + "'" ,
			webRoot = "'" + basePath + "'" ,
			scriptPath = "'" + sPath + "'" ) ) )

	return apache.OK
